@@ -10,8 +10,13 @@
 #'   class \code{"internalImage"}, which contains only minimal metadata about
 #'   the image. Either way, the return value has an attribute which points to a
 #'   C data structure containing the full image.
-#' @return An array or internal image, with class \code{"niftiImage"}, and
-#'   possibly also \code{"internalImage"}.
+#' @param volumes An integer vector giving the volumes to read (counting along
+#'   all dimensions beyond the third jointly), or \code{NULL}, the default, in
+#'   which case every volume is read. This cannot currently be set differently
+#'   for each file read.
+#' @return An array or internal image, with class \code{"niftiImage"} (and
+#'   possibly also \code{"internalImage"}), or a list of such objects if
+#'   \code{file} has length greater than one.
 #' 
 #' @note If the \code{internal} argument is \code{FALSE} (the default), the
 #'   data type of the image pointer will be set to match one of R's native
@@ -30,16 +35,16 @@
 #' @seealso \code{\link{writeNifti}}
 #' @references The NIfTI-1 standard (\url{http://www.nitrc.org/docman/view.php/26/64/nifti1.h}).
 #' @export
-readNifti <- function (file, internal = FALSE)
+readNifti <- function (file, internal = FALSE, volumes = NULL)
 {
     if (!is.character(file))
         stop("File name(s) must be specified in a character vector")
     if (length(file) == 0)
         stop("File name vector is empty")
     else if (length(file) > 1)
-        lapply(file, function(f) .Call("readNifti", path.expand(f), internal, PACKAGE="RNifti"))
+        lapply(file, function(f) .Call("readNifti", path.expand(f), internal, volumes, PACKAGE="RNifti"))
     else
-        .Call("readNifti", path.expand(file), internal, PACKAGE="RNifti")
+        .Call("readNifti", path.expand(file), internal, volumes, PACKAGE="RNifti")
 }
 
 #' Write a NIfTI-1 format file
@@ -94,7 +99,7 @@ writeNifti <- function (image, file, template = NULL, datatype = "auto")
 #' @export
 retrieveNifti <- function (object)
 {
-    .Call("readNifti", object, TRUE, PACKAGE="RNifti")
+    .Call("readNifti", object, TRUE, NULL, PACKAGE="RNifti")
 }
 
 #' Update an internal NIfTI-1 object using a template
@@ -103,19 +108,46 @@ retrieveNifti <- function (object)
 #' using metadata from the template. The dimensions and, if available, pixel
 #' dimensions, from the \code{image} will replace those from the template.
 #' 
+#' If \code{template} is a complete list of NIfTI-1 header fields, like that
+#' produced by \code{\link{dumpNifti}}, or an image, then it will be used to
+#' create the internal object, and then the data and metadata associated with
+#' the \code{image} will overwrite the appropriate parts. If \code{template}
+#' is an incomplete list, the \code{image} will be used to create the internal
+#' object, and then the specified fields will be overwritten from the list.
+#' This allows users to selectively update certain fields while leaving others
+#' alone (but see the note below).
+#' 
+#' Datatype information in a list \code{template} is ignored. The datatype can
+#' only be changed using the \code{datatype} argument, but in this case the
+#' internal object gets out of sync with the R array, so an internal image is
+#' returned to avoid the mismatch. Changing the internal datatype in this way
+#' is for advanced usage only.
+#' 
 #' @param image A numeric array.
 #' @param template An image, in any acceptable form (see
 #'   \code{\link{retrieveNifti}}), or a named list of NIfTI-1 properties like
 #'   that produced by \code{\link{dumpNifti}}. The default of \code{NULL} will
 #'   have no effect.
+#' @param datatype The NIfTI datatype to use within the internal image. The
+#'   default, \code{"auto"} uses the R type. Other possibilities are
+#'   \code{"float"}, \code{"int16"}, etc., which may be preferred to reduce
+#'   object size. However, no checks are done to ensure that the coercion
+#'   maintains precision, and this option is for advanced usage only.
 #' @return A copy of the original \code{image}, with its internal image
-#'   attribute set or updated appropriately.
+#'   attribute set or updated appropriately. If \code{datatype} is not
+#'   \code{"auto"} then the result is an internal image.
+#' 
+#' @note The \code{scl_slope} and \code{scl_inter} fields affect the numerical
+#'   interpretation of the pixel data, so it is impossible in general to change
+#'   them without also changing the array values on both the C and the R side.
+#'   Therefore, to avoid unexpected side-effects, these fields are not affected
+#'   by this function.
 #' 
 #' @author Jon Clayden <code@@clayden.org>
 #' @export
-updateNifti <- function (image, template = NULL)
+updateNifti <- function (image, template = NULL, datatype = "auto")
 {
-    .Call("updateNifti", image, template, PACKAGE="RNifti")
+    .Call("updateNifti", image, template, datatype, PACKAGE="RNifti")
 }
 
 #' Dump the contents of an internal NIfTI-1 object
@@ -156,6 +188,33 @@ print.niftiHeader <- function (x, ...)
         cat(paste0(paste(rep(" ",maxWidth-widths[i]),collapse=""), names(x)[i], ": ", paste(format(x[[i]],trim=TRUE),collapse="  "), "\n"))
 }
 
+#' Check the format version of a file
+#' 
+#' This function identifies the likely NIfTI format variant used by one or more
+#' files on disk.
+#' 
+#' @param file A character vector of file names.
+#' @return A vector of integers, of the same length as \code{file}. Each
+#'   element will be 0 for ANALYZE format (the precursor to NIfTI-1), 1 for
+#'   NIfTI-1 (which is now most common), 2 for NIfTI-2, or -1 if the file
+#'   doesn't exist or doesn't look plausible in any of these formats.
+#' 
+#' @note NIfTI-2 format, mostly a variant of NIfTI-1 with wider datatypes used
+#'   for many fields, is not currently supported for reading, but it is
+#'   detected by this function.
+#' 
+#' @examples
+#' path <- system.file("extdata", "example.nii.gz", package="RNifti")
+#' niftiVersion(path)       # 1
+#' 
+#' @author Jon Clayden <code@@clayden.org>
+#' @seealso \code{\link{readNifti}}
+#' @export
+niftiVersion <- function (file)
+{
+    sapply(file, function(f) .Call("niftiVersion", path.expand(f), PACKAGE="RNifti"))
+}
+
 rescaleNifti <- function (image, scales)
 {
     .Call("rescaleImage", image, scales, PACKAGE="RNifti")
