@@ -1,8 +1,8 @@
 var indexSectionsWithContent =
 {
-  0: "bcdghinoprstuvx~",
+  0: "bcdfginoprstuvx~",
   1: "bn",
-  2: "bcdghinoprstuvx~",
+  2: "bcdfginoprstuvx~",
   3: "dip",
   4: "r"
 };
